/* debug.c: a SCSI protocol dumper called from do_scsi_command
 * Copyright (C) 1996 Michael K. Johnson
 * 
 * I can be reached via email at johnsonm@redhat.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 * 
 */

#include "debug.h"

typedef struct {
  int size;
  char *name;
} cmnds;

cmnds cds[] = {
  {6, "TEST_UNIT_READY"},
  {0, ""},
  {0, ""},
  {6, "REQUEST_SENSE"},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {6, "INQUIRY"},
  {0, ""},
  {0, ""},
  {0, ""},
  {6, "RESERVE_UNIT"},
  {6, "RELEASE_UNIT"},
  {0, ""},
  {0, ""},
  {0, ""},
  {6, "SCAN"},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {10, "SET_WINDOW"},
  {0, ""},
  {0, ""},
  {0, ""},
  {10, "READ"},
  {0, ""},
  {10, "SEND"},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {10, "OBJECT_POSITION"},
  {0, ""},
  {0, ""},
  {10, "GET_DATA_BUFFER_STATUS"},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {0, ""},
  {10, "WRITE_BUFFER"},
  {0, ""},
  {0, ""},
};

static inline char to_hex(int i) {
  static char hextab[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                           'a', 'b', 'c', 'd', 'e', 'f' };
  return hextab[i];
}


/* ========================================================================= */


#define CL(b) (memset(b, '\000', sizeof(b)))
#define WH(a) (memset(a, ' ', sizeof(a)-1))


/* ========================================================================= */


void dump_send (char *b, int len, int f)
{
  unsigned char *buf = (unsigned char *) b;
  int command;
  int i, j=0;
  char hex_buffer[80];
  int hc;
  char ascii_buffer[17];
  int ac;

  WH(ascii_buffer); ascii_buffer[16] = '\n';
  CL(hex_buffer); hc = 0;

  write(f, "\nDATA OUT:\n", 11);
  command = (int) buf[0];

  if (command > (sizeof(cds)/sizeof(cmnds)) || cds[command].size == 0)
  {
    sprintf(hex_buffer, "Command unknown: %d\n%n", command, &hc);
    write(f, hex_buffer, hc);
    CL(hex_buffer); hc = 0;
  }
  else
  {
    int k;
    for (i=0, hc=0, ac=0; i < cds[command].size; i++)
    {
      /* Print a byte */
      hex_buffer[hc++] = to_hex(buf[i] >> 4);
      hex_buffer[hc++] = to_hex(buf[i] & 0x0f);
      hex_buffer[hc++] = ' ';

      /* And format it in ascii */
      if (isprint(buf[i])) { ascii_buffer[ac++] = buf[i]; }
      else { ascii_buffer[ac++] = '.'; }
    }

    /* Now print name of command */
    sprintf(&hex_buffer[hc], " Command: %s   %n", cds[command].name, &i);
    hc += i;
    /* pad the output to make it look nice */
    if (hc < 49)
    {
      int l = 49 - hc;
      int c;
      for (c=0; c<l; c++) { hex_buffer[hc++] = ' '; }
    }
    write(f, hex_buffer, hc);
    CL(hex_buffer); hc=0;

    /* output ascii buffer */
    write(f, ascii_buffer, 17);
    WH(ascii_buffer); ac=0;
  }


  i=cds[command].size; /* from oliver rauch */
  if (i < len)
  {
    /* Only if there is data to print, print data-out phase */
    for (j=1, hc=0, ac=0 ; i < len; i++, j++)
    {

      /* write the hex version */
      hex_buffer[hc++] = to_hex(buf[i] >> 4);
      hex_buffer[hc++] = to_hex(buf[i] & 0x0f);
      hex_buffer[hc++] = ' ';
      if (j == 8) hex_buffer[hc++] = ' ';

      /* And format it in ascii */
      if (isprint(buf[i])) { ascii_buffer[ac++] = buf[i]; }
      else { ascii_buffer[ac++] = '.'; }

      if (j >= 16)
      {
	j = 0;
	write(f, hex_buffer, hc);
        CL(hex_buffer);
	if (i < len) hc = 0;
	write(f, ascii_buffer, 17);
        WH(ascii_buffer); ac = 0;
      }
    }

    /* Finish off */
    if (j)
    {
      /* pad the output to make it look nice */
      if (hc < 49)
      {
        int l = 49 - hc;
        int c;
        for (c=0; c<l; c++)
	{ hex_buffer[hc] = ' '; hc++; }
      }
      write(f, hex_buffer, hc);
      write(f, ascii_buffer, 17);
    }

  }
}


/* ========================================================================= */


void dump_receive (char *b, int len, unsigned int res, int f)
{
  unsigned char *buf = (unsigned char *) b;
  int i, j;
  char hex_buffer[60];
  int hc;
  char ascii_buffer[17];
  int ac;
  
  /* just dump data-in */
  sprintf(hex_buffer, "DATA IN:\nResponse code: %2.2x\n%n", res, &hc);
  write(f, hex_buffer, hc);
  CL(hex_buffer); hc = 0; 
  WH(ascii_buffer); ascii_buffer[16] = '\n'; ac = 0;
  for (i=0, j=1 ; i < len; i++, j++)
  {
    /* write the hex version */
    hex_buffer[hc++] = to_hex(buf[i] >> 4);
    hex_buffer[hc++] = to_hex(buf[i] & 0x0f);
    hex_buffer[hc++] = ' ';
    if (j == 8) hex_buffer[hc++] = ' ';
    /* And format it in ascii */
    if (isprint(buf[i])) { ascii_buffer[ac++] = buf[i]; }
    else { ascii_buffer[ac++] = '.'; }
    if (j >= 16)
    {
      j = 0;
      write(f, hex_buffer, hc);
      CL(hex_buffer);
      if (i < len) hc = 0;
      write(f, ascii_buffer, 17);
      WH(ascii_buffer); ac = 0;
    }
  }
  /* Finish off */
  if (j)
  {
    /* pad the output to make it look nice */
    if (hc < 49)
    {
      int l = 49 - hc;
      int c;
      for (c=0; c<l; c++)
      { hex_buffer[hc] = ' '; hc++; }
    }
    write(f, hex_buffer, hc);
    write(f, ascii_buffer, 17);
  }
}
