/* ------------------------------------------------------------------------- */

/* umax-scsi.h: header-file for low-level-driver for UMAX scanners
  
   Copyright (C) 1996-1997 Michael K. Johnson
   Copyright (C) 1997 Oliver Rauch

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   As a special exception, the authors of SANE give permission for
   additional uses of the libraries contained in this release of SANE.

   The exception is that, if you link a SANE library with other files
   to produce an executable, this does not by itself cause the
   resulting executable to be covered by the GNU General Public
   License.  Your use of that executable is in no way restricted on
   account of linking the SANE library code into it.

   This exception does not, however, invalidate any other reasons why
   the executable file might be covered by the GNU General Public
   License.

   If you submit changes to SANE to the maintainers to be included in
   a subsequent release, you agree by submitting the changes that
   those changes may be distributed with this exception intact.

   If you write modifications of your own for SANE, it is your choice
   whether to permit this exception to apply to your modifications.
   If you do not wish that, delete this exception notice.
  
 */

/* ------------------------------------------------------------------------- */

#ifndef UMAX_SCSI_H
#define UMAX_SCSI_H

#include "umax-struct.h"

int    umax_do_scsi_open(char * device, int * sfd, void * handler);
int    umax_do_scsi_close(int sfd);
int    umax_do_scsi_cmd(int fd,char * cmd,int cmd_len, char * out, int out_len);
int    umax_get_data_buffer_status(US *us);
void   umax_do_request_sense(US *us);
int    umax_test_warmup(US *us);
int    umax_wait_scanner(US *us);
int    umax_emergency_give_scanner(int i);
int    umax_grab_scanner(US *us);
int    umax_reposition_scanner(US *us);
int    umax_give_scanner(US *us);
void   umax_send_gamma_data(US *us, char *data, int color);
int    umax_correct_light(int light, int analog_gamma_byte);
int    umax_set_window_param(US *us);
int    umax_start_scan(US *us);
int    umax_read_data_block(US *us, unsigned int length);
void   umax_do_inquiry(US *us);
int    umax_identify_scanner(US *us);
int    umax_open_scanner(US *us);
void   umax_trim_rowbufsize(US *us);
int    umax_bound_scanarea(US *us);
void   umax_calculate_exposure_time(US *us, int def,int *value);
int    umax_check_values(US *us);
void   umax_get_inquiry_values(US *us);
int    umax_calculate_analog_gamma(double value);
double umax_get_analog_gamma(int value);
void   umax_initialize_values(US *us);
void   umax_init(US *us);

#define WAIT_SCANNER ret = umax_wait_scanner(us); if (ret) return ret

#endif /* UMAX_SCSI_H */

