/* ------------------------------------------------------------------------- */


/* umax-text.c: textoutput of driver for UMAX scanners.
  
   (C) 1997 Oliver Rauch
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version
   2 of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public
   License along with this program ( thats a lie at the moment)
   if not, write to the Free Software Foundation, Inc.,
   675 Mass Ave, Cambridge, MA 02139, USA.
   
 */


/* ------------------------------- INCLUDES -------------------------------- */

#include <stdio.h>
#include <scsi/scsi.h>
#include <scsi/sg.h>

#include "scsicmd.h"           
#include "umax-struct.h"
#include "umax-scsi.h"
#include "umax-scsidef.h"
#include "debug.h"
#include "umax-text.h"


/* --------------------------------- DBG ----------------------------------- */

int debug_level;

#undef DBG
#define DBG(level,msg, args...) \
{ if (level <= debug_level ) { fprintf(stderr, msg, ##args); }

/* ------------------------------ DBG_INIT --------------------------------- */

#undef DBG_INIT
#define DBG_INIT() { debug_level = 1; }

/* --------------------------- PRINT NOTZERO ------------------------------- */


void print_notzero(char *text, int flag)
{ if (flag != 0) { fprintf(stderr,"%s",text); } }


/* ----------------------------- PRINT SENSE ------------------------------- */


void print_sense(unsigned char *rsr_block)
{
int len,sensekey,asc,ascq,asc_ascq;

   if (get_RS_error_code(rsr_block) != 0x70)
   {
      fprintf(stderr,"request sense did not return error code (0x70)\n");
      return;
   }

   len=7+get_RS_additional_length(rsr_block);

   if (get_RS_ILI(rsr_block) != 0)
   {
      fprintf(stderr, "ILI-ERROR: requested data length"
      " is larger than actual length on the medium\n");
   }

   sensekey=get_RS_sense_key(rsr_block);
   fprintf(stderr, "%s\n",sense_str[sensekey]);

   if (sensekey == 0) {return;}

   if (len <= 0x0d) {return;};
   asc      = get_RS_ASC(rsr_block);
   ascq     = get_RS_ASCQ(rsr_block);
   asc_ascq = 256*asc+ascq;


   switch (sensekey)
   {
    case 0x03: /* medium error */
      if (asc_ascq == 0x1400)
      { fprintf(stderr,"misfeed, paper jam\n"); }
      else if (asc_ascq == 0x1401)
      { fprintf(stderr,"adf not ready\n"); }
      else
      { fprintf(stderr,"unknown error: asc=%d, ascq=%d\n",asc,ascq); }
      break;

    case 0x04: /* hardware error */
      if (asc_ascq == 0x4000) { fprintf(stderr,"diagnostic error:"); }

      if (len <= 0x13) {return;};
      print_notzero("dim_light\n",get_RS_asb_dim_light(rsr_block));
      print_notzero("no light\n",get_RS_asb_no_light(rsr_block));
      print_notzero("sensor or motor error\n",
                     get_RS_asb_sensor_motor(rsr_block));
      print_notzero("too light\n",get_RS_asb_too_light(rsr_block));
      print_notzero("calibration error\n",get_RS_asb_calibration(rsr_block));
      print_notzero("rom error\n",get_RS_asb_rom(rsr_block));
      print_notzero("ram error\n",get_RS_asb_ram(rsr_block));
      print_notzero("cpu error\n",get_RS_asb_cpu(rsr_block));
      print_notzero("scsi error\n",get_RS_asb_scsi(rsr_block));
      print_notzero("timer error\n",get_RS_asb_timer(rsr_block));
      print_notzero("filter motor error\n",get_RS_asb_filter_motor(rsr_block));
      print_notzero("dc adjust error\n",get_RS_asb_dc_adjust(rsr_block));
      print_notzero("uta home sensor or motor error\n",
                     get_RS_asb_uta_sensor(rsr_block));
      break;

    case 0x05: /* illegal request */
      if (asc_ascq == 0x2000)
      { fprintf(stderr,"invalid command operation code\n"); }
      else if (asc_ascq == 0x2400)
      { fprintf(stderr,"illegal field in CDB\n"); }
      else if (asc_ascq == 0x2500)
      { fprintf(stderr,"logical unit not supported\n"); }
      else if (asc_ascq == 0x2600)
      { fprintf(stderr,"invalid field in parameter list\n"); }
      else if (asc_ascq == 0x2c01)
      { fprintf(stderr,"too many windows specified\n"); }
      else if (asc_ascq == 0x2c02)
      { fprintf(stderr,"invalid combination of windows specified\n"); }
      else
      { fprintf(stderr,"unknown error: asc=%d, ascq=%d\n",asc,ascq);}

      if (len <= 0x11) {return;};
      if (get_RS_SKSV(rsr_block) != 0)
      {
         if (get_RS_CD(rsr_block) == 0)
	 { fprintf(stderr,"illegal parameter in CDB\n"); }
	 else
	 {
            fprintf(stderr, "illegal parameter is in the data"
                           "parameters sent during data out phase\n");
         }

	 fprintf(stderr,"error detected in byte %d\n",
                        get_RS_field_pointer(rsr_block));
      }
      break;

    case 0x06: /* unit attention */
      if (asc_ascq == 0x2900)
      { fprintf(stderr,"power on, reset or bus-device error\n"); }
      else if (asc_ascq == 0x3f01)
      { fprintf(stderr,"microcode has been changed\n"); }
      else
      { fprintf(stderr,"unknown error: asc=%d, ascq=%d\n",asc,ascq);}
      break;
      
    case 0x09: /* vendor specific */
      if (asc_ascq == 0x8001)
      { fprintf(stderr,"lamp warmup\n"); }
      else if (asc_ascq == 0x8002)
      { fprintf(stderr,"calibration by driver\n"); }
      else
      { fprintf(stderr,"unknown error: asc=%d, ascq=%d\n",asc,ascq);}
      break;

    default:
      fprintf(stderr,"unknown error: sensekey=%d, asc=%d, ascq=%d\n",
                      sensekey,asc,ascq);
   }

   if (len <= 0x15) {return;};
   fprintf(stderr,"%s\n",
           scanner_error_str[get_RS_scanner_error_code(rsr_block)]);
}


/* ============================ PRINT_INQUIRY ======================== */


void print_inquiry(US *us)
{
char * inquiry_block;


  inquiry_block=us->buffer;


  fprintf(stderr,"INQUIRY:\n");
  fprintf(stderr,"========\n\n");
  fprintf(stderr,"vendor........................: '%s'\n",us->vendor);
  fprintf(stderr,"product.......................: '%s'\n",us->product);
  fprintf(stderr,"version.......................: '%s'\n",us->version);

  fprintf(stderr,"\nCBHS value range..............: %s\n",
                 cbhs_str[us->inquiry_cbhs]);
  fprintf(stderr,"scanmode......................: %s\n",
                  scanmode_str[get_inquiry_scanmode(inquiry_block)]);
  if (us->inquiry_transavail)
   {
        fprintf(stderr,"UTA (transparency).............: available");
        print_notzero(",lamp status is true",
                      get_inquiry_translamp(inquiry_block));
	fprintf(stderr,"\n");
   }

  fprintf(stderr,"inquiry block lenght..........: %d bytes\n",us->inquiry_len);
  if (us->inquiry_len<=0x8e)
  { fprintf(stderr,
       "Inquiry block is unexpected short, should be at least 147 bytes\n"); } 


  /* 0x24 */
  if (us->inquiry_len<=0x24) {return;}
  fprintf(stderr,"\nf/w support function:\n");
  fprintf(stderr,"---------------------\n");
  print_notzero("quality calibration\n", us->inquiry_quality_ctrl);
  print_notzero("fast preview function\n", us->inquiry_preview);
  print_notzero("shadow compensation by f/w\n",
                get_inquiry_fw_shadow_comp(inquiry_block));
  print_notzero("reselection phase\n",
                get_inquiry_fw_reselection(inquiry_block));
  print_notzero("lamp intensity control\n", us->inquiry_lamp_ctrl);
  print_notzero("batch scan function\n",
                get_inquiry_fw_batch_scan(inquiry_block));
  print_notzero("calibration mode control by driver\n",
                get_inquiry_fw_calibration(inquiry_block));

  if (get_inquiry_fw_adjust_exposure_tf(inquiry_block))
  {
    int unit;

    fprintf(stderr,"\nadjust exposure time function\n");
    unit=get_inquiry_exposure_time_step_unit(inquiry_block);
    fprintf(stderr,"exposure time step units......: %d micro-sec\n",unit);
    fprintf(stderr,"exposure time maximum.........: %d micro-sec\n",
            unit*get_inquiry_exposure_time_step_unit(inquiry_block));
    fprintf(stderr,"exposure time minimum (LHG)...: %d micro-sec\n",
            unit*get_inquiry_exposure_time_lhg_min(inquiry_block));
    fprintf(stderr,"exposure time minimum color...: %d micro-sec\n",
            unit*get_inquiry_exposure_time_color_min(inquiry_block));
    fprintf(stderr,"exposure time default FB (LH).: %d micro-sec\n",
            unit*get_inquiry_exposure_time_lh_def_fb(inquiry_block));
    fprintf(stderr,"exposure time default UTA (LH): %d micro-sec\n",
            unit*get_inquiry_exposure_time_lh_def_uta(inquiry_block));
    fprintf(stderr,"exposure time default FB grey.: %d micro-sec\n",
            unit*get_inquiry_exposure_time_grey_def_fb(inquiry_block));
    fprintf(stderr,"exposure time default UTA grey: %d micro-sec\n",
            unit*get_inquiry_exposure_time_grey_def_uta(inquiry_block));
    fprintf(stderr,"exposure time default FB red..: %d micro-sec\n",
            unit*get_inquiry_exposure_time_def_r_fb(inquiry_block));
    fprintf(stderr,"exposure time default FB grn..: %d micro-sec\n",
            unit*get_inquiry_exposure_time_def_g_fb(inquiry_block));
    fprintf(stderr,"exposure time default FB blue.: %d micro-sec\n",
            unit*get_inquiry_exposure_time_def_b_fb(inquiry_block));
    fprintf(stderr,"exposure time default UTA red.: %d micro-sec\n",
            unit*get_inquiry_exposure_time_def_r_uta(inquiry_block));
    fprintf(stderr,"exposure time default UTA grn.: %d micro-sec\n",
            unit*get_inquiry_exposure_time_def_g_uta(inquiry_block));
    fprintf(stderr,"exposure time default UTA blue: %d micro-sec\n",
            unit*get_inquiry_exposure_time_def_b_uta(inquiry_block));
  }


  /* 0x60 */
  if (us->inquiry_len<=0x60) {return;}
  fprintf(stderr,"\nscan modes:\n");
  fprintf(stderr,"-----------\n");
  print_notzero("three passes color mode\n",
                get_inquiry_sc_three_pass_color(inquiry_block));
  print_notzero("single pass color mode\n",
                get_inquiry_sc_one_pass_color(inquiry_block));
  print_notzero("lineart mode\n",us->inquiry_lineart);
  print_notzero("halftone mode\n",us->inquiry_halftone);
  print_notzero("grey mode\n",us->inquiry_grey);
  print_notzero("color mode\n",us->inquiry_color);
  print_notzero("transparency (UTA)\n",us->inquiry_uta);
  print_notzero("automatic document feeder (ADF)\n", us->inquiry_adf);

  /* 0x61 */
  if (us->inquiry_len<=0x61) {return;}
  fprintf(stderr,"\nscanner capability:\n");
  fprintf(stderr,"-------------------\n");
  print_notzero("double resolution\n", us->inquiry_dor);
  print_notzero("send high byte first\n",
                get_inquiry_sc_high_byte_first(inquiry_block));
  print_notzero("bi-level image reverse\n", us->inquiry_reverse);
  print_notzero("multi-level image reverse\n", us->inquiry_reverse_multi);
  print_notzero("support shadow function\n", us->inquiry_shadow);
  print_notzero("support highlight function\n",us->inquiry_highlight);
  print_notzero("f/w downloadable\n",
                get_inquiry_sc_downloadable_fw(inquiry_block));
  print_notzero("paper length can reach to 14 inch\n",
                get_inquiry_sc_paper_length_14(inquiry_block));

  /* 0x62 */
  if (us->inquiry_len<=0x62) {return;}
  print_notzero("shading data/gain uploadable\n",
                get_inquiry_sc_uploadable_shade(inquiry_block));
  print_notzero("analog gamma correction\n", us->inquiry_analog_gamma);
  print_notzero("x,y coordinate base\n",
                get_inquiry_xy_coordinate_base(inquiry_block));

  /* 0x64 - 0x68*/
  if (us->inquiry_len<=0x68) {return;}
  if (us->inquiry_gamma_dwload)
   {
       fprintf(stderr,"\ngamma download available\n");
       print_notzero("gamma download type 2\n",
                     get_inquiry_gamma_type(inquiry_block));
       fprintf(stderr,"lines of gamma curve..........: %s\n",
               gamma_lines_str[get_inquiry_gamma_lines(inquiry_block)]);

       /* 0x66 */
       print_notzero("gamma input   8 bits/pixel support\n",
                     get_inquiry_gib_8bpp(inquiry_block));
       print_notzero("gamma input  10 bits/pixel support\n",
                     get_inquiry_gib_10bpp(inquiry_block));

       /* 0x68 */
       print_notzero("gamma output  8 bits/pixel support\n",
                      get_inquiry_gob_8bpp(inquiry_block));
       print_notzero("gamma output 10 bits/pixel support\n",
                      get_inquiry_gob_10bpp(inquiry_block));
   }

  /* 0x69 */
  if (us->inquiry_len<=0x69) {return;}
  fprintf(stderr,"\n");
  if (get_inquiry_hda(inquiry_block))
   {
       fprintf(stderr,"halftone download available\n");
       fprintf(stderr,"halftone pattern download max matrix %dx%d\n",
                       get_inquiry_max_halftone_matrix(inquiry_block),
                       get_inquiry_max_halftone_matrix(inquiry_block));
   }

  /* 0x6a */
  if (us->inquiry_len<=0x6a) {return;}
  print_notzero("built-in halftone pattern.....: ",
                 get_inquiry_halftones_supported(inquiry_block));
  print_notzero("2x2 ",get_inquiry_halftones_2x2(inquiry_block));
  print_notzero("4x4 ",get_inquiry_halftones_4x4(inquiry_block));
  print_notzero("6x6 ",get_inquiry_halftones_6x6(inquiry_block));
  print_notzero("8x8 ",get_inquiry_halftones_8x8(inquiry_block));
  print_notzero("12x12 ",get_inquiry_halftones_12x12(inquiry_block));
  print_notzero("\n",get_inquiry_halftones_supported(inquiry_block));

  /* 0x6d */
  if (us->inquiry_len<=0x6d) {return;}
  fprintf(stderr,"\ncolor sequence................: %s\n",
          color_sequence_str[get_inquiry_colorseq(inquiry_block)]);
  fprintf(stderr,"color ordering support........: ");
  print_notzero("(pixel) ",get_inquiry_color_order_pixel(inquiry_block));
  print_notzero("(line) ",get_inquiry_color_order_line_no_ccd(inquiry_block));
  print_notzero("(plane) ",get_inquiry_color_order_plane(inquiry_block));
  print_notzero("(line with ccd distance)",
                get_inquiry_color_order_line_w_ccd(inquiry_block));
  fprintf(stderr,"\n");

  /* 0x6e */
  if (us->inquiry_len<=0x71) {return;}
  fprintf(stderr,"\nmaximum video memory..........: %d KB\n",
          get_inquiry_max_vidmem(inquiry_block)/1024);

  /* 0x73 */ /* 0x94 */
  if (us->inquiry_len<=0x73) {return;}
  fprintf(stderr,"maximum optical resolution....: %d dpi\n",
          us->inquiry_optical_res);

  /* 0x74 */ /* 0x95 */
  if (us->inquiry_len<=0x74) {return;}
  fprintf(stderr,"maximum x-resolution..........: %d dpi\n",
          us->inquiry_x_res);

  /* 0x75 */ /* 0x96 */
  if (us->inquiry_len<=0x75) {return;}
  fprintf(stderr,"maximum y-resolution..........: %d dpi\n",
          us->inquiry_y_res);

  /* ---------- */

  fprintf(stderr,"\nFB (flatbed-mode):\n");
  /* 0x76 */
  if (us->inquiry_len<=0x76) {return;}
  fprintf(stderr,"FB maximum scan width.........: %2.2f inch\n",
          us->inquiry_fb_width);
  /* 0x77 */
  if (us->inquiry_len<=0x77) {return;}
  fprintf(stderr,"FB maximum scan length........: %2.2f inch\n",
          us->inquiry_fb_length);

  /* ---------- */

  if (us->inquiry_len<=0x81) {return;}
  fprintf(stderr,"\nUTA (transparency-mode):\n");
  /* 0x7a */
  fprintf(stderr,"UTA x-original point..........: %2.2f inch\n",
          (double)get_inquiry_uta_x_original(inquiry_block)*0.01);
  /* 0x7c */
  fprintf(stderr,"UTA y-original point..........: %2.2f inch\n",
          (double)get_inquiry_uta_y_original(inquiry_block)*0.01);

  /* 0x7e */
  fprintf(stderr,"UTA maximum scan width........: %2.2f inch\n",
          us->inquiry_uta_width);
  /* 0x80 */
  fprintf(stderr,"UTA maximum scan length.......: %2.2f inch\n",
          us->inquiry_uta_length);
 
  /* ---------- */
  
  if (us->inquiry_len<=0x8d) {return;}
  fprintf(stderr,"\nDOR (double optical resolution-mode):\n");
  /* 0x86 */
  fprintf(stderr,"DOR x-original point..........: %2.2f inch\n",
          (double)get_inquiry_dor_x_original_point(inquiry_block)*0.01);
  /* 0x88 */
  fprintf(stderr,"DOR y-original point..........: %2.2f inch\n",
          (double)get_inquiry_dor_y_original_point(inquiry_block)*0.01);

  /* 0x8a */
  fprintf(stderr,"DOR maximum scan width........: %2.2f inch\n",
          us->inquiry_dor_width);
  /* 0x8c */
  fprintf(stderr,"DOR maximum scan length.......: %2.2f inch\n",
          us->inquiry_dor_length);
  fprintf(stderr,"\n");

  /* ---------- */

  /* 0x8f */
  if (us->inquiry_len<=0x8f) {return;}
  fprintf(stderr,"last calibration lamp density : %d\n",
          get_inquiry_last_calibration_lamp_density(inquiry_block));
  /* 0x91 */
  if (us->inquiry_len<=0x91) {return;}
  fprintf(stderr,"lamp warmup maximum time......: %d sec\n",
          us->inquiry_max_warmup_time);
 
  /* 0x92 */
  if (us->inquiry_len<=0x93) {return;}
  fprintf(stderr,"window descriptor block length: %d bytes\n",
          get_inquiry_wdb_length(inquiry_block));

  /* ----------------- */

  /* 0x97 */
  if (us->inquiry_len<=0x97) {return;}
  if (get_inquiry_analog_gamma_table(inquiry_block) == 0)
    { fprintf(stderr,"no analog gamma function\n"); }
  else
    { fprintf(stderr,"mp 8832 analog gamma table\n"); }

  /* 0x9a */
  if (us->inquiry_len<=0x9a) {return;}
  fprintf(stderr,"maximum calibration data lines for shading %d\n",
          get_inquiry_max_calibration_data_lines(inquiry_block));

};


/* -------------------------- PRINT VARIABLES ------------------------------ */


void print_variables(US *us)
{
 fprintf(stderr,
  "Size-, position- and color-related variables:\n"
  "row_len          = %u (measured in bytes)\n"
  "width_in_pixels  = %u\n"
  "height_in_pixels = %u\n"
  "phys_width       = %u (1200pt/inch)\n"
  "phys_height      = %u (1200pt/inch)\n"
  "x_resolution     = %u (pixels/inch)\n"
  "y_resolution     = %u (pixels/inch)\n"
  "upper_left_x     = %u (offset 1200pt/inch)\n"
  "upper_left_y     = %u (offset 1200pt/inch)\n"
  "brightness       = %u\n"
  "threshold        = %u\n"
  "contrast         = %u\n"
  "highlight        = %u %u %u\n"
  "shadow           = %u %u %u\n"
  "digital gamma    = %1.4f %1.4f %1.4f\n"
  "analog gamma     = %1.4f %1.4f %1.4f\n"
  "\n",
  us->row_len, us->width_in_pixels, us->height_in_pixels,
  us->phys_width, us->phys_height,
  us->x_resolution, us->y_resolution,
  us->upper_left_x, us->upper_left_y,
  us->brightness, us->threshold, us->contrast,
  us-> highlight_r, us-> highlight_g, us-> highlight_b,
  us->shadow_r, us->shadow_g, us->shadow_b,
  us->gamma_r, us->gamma_g, us->gamma_b,
  umax_get_analog_gamma(us->analog_gamma_r),
  umax_get_analog_gamma(us->analog_gamma_g),
  umax_get_analog_gamma(us->analog_gamma_b)
  );
}

